/*****************************************************************************/
/*                                                                           */
/*      Version:  0.99   Date: 27/01/09   File: first_aco_code.c             */
/* Last Version:                          File:                              */
/* Changes:                                                                  */
/*                                                                           */
/* Purpose:  template to be used for first ACO code                          */
/*                                                                           */
/*                                                                           */
/* Author:  Thomas Stuetzle                                                  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*===========================================================================*/

#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

#define PROG_ID_STR "Ant System for TSP, V0.99999\n"
#define CALL_SYNTAX_STR "aco_code distance_matrix_file\n" 

#define VERBOSE(x) x

#define MAX(x,y) ((x)>=(y)?(x):(y))

#define FALSE 0
#define TRUE  1

/* constants for a pseudo-random number generator, details see Numerical Recipes in C book */

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836

/* seed for the random number generator */

long int seed = -1; /* MAO: initialized to -1, meaning "uninitialized"... */

long int opt;

long int distance[1024][1024];    /* distance matrix; static definition */

double   pheromone[1024][1024];   /* pheromone matrix; static definition */

double   combined[1024][1024];    /* combined value of pheromone X heuristic information */

typedef struct {
  long int tour[1024];           /* solution */
  long int visited[1024];        /* auxiliary array for construction to keep track of visited cities */
  long int tour_length;
} ant_struct;

ant_struct colony[1024];

long int n_cities;                       /* number of cities in the TSP instance */
long int shortest_found_tour_length;     /* length of the shortest tour found */

long int max_trials;           /* max. number of independent repetitions of the ACO code */
long int trials;               /* counter for the number of trials */

long int max_iterations;       /* max. number of iterations of each trial of the ACO algorithm*/
long int iterations;           /* counter for the number of iterations */


double   rho;                  /* parameter for pheromone trail evaporation */
long int n_ants;               /* number of ants in the colony */ 



double ran01( long *idum ) {
/* 
      FUNCTION:      returns a pseudo-random number
      INPUT:         a pointer to the seed variable 
      OUTPUT:        a pseudo-random number uniformly distributed in [0,1]
      (SIDE)EFFECTS: changes the value of seed
      Notes: call this function using ran01(&seed)
*/
  long k;
  double ans;

  k =(*idum)/IQ;
  *idum = IA * (*idum - k * IQ) - IR * k;
  if (*idum < 0 ) *idum += IM;
  ans = AM * (*idum);
  return ans;
}



void read_problem_size( FILE *input ) {
/* 
      FUNCTION:      read the dimension of the TSP instance
      INPUT:         a pointer to the input file 
      OUTPUT:        none (void)
      (SIDE)EFFECTS: assigns n_cities the instance dimension (= number of cities)
*/
  if( fscanf(input, "%ld", &n_cities) <= 0) {
    fprintf(stderr, "error reading tsp size value in data file\n");
    exit(1);	
  }
  VERBOSE ( printf("TSP instance size %ld\n",n_cities); )
}



void read_distance_matrix( FILE *input, long int size )
/*    
      FUNCTION:      read a TSP distance matrix from the input file
      INPUT:         Pointer to input file, number cities of TSP instance
      OUTPUT:        none (void)
      (SIDE)EFFECTS: assigns distance values to submatrix 0 .. n_cities X 0 .. n_cities of distance[][]
*/
{
  long int     i, j;

  for ( i = 0 ; i < size ; i++ ) {
    for ( j = 0  ; j < size ; j++ ) {
      if( fscanf(input, "%ld", &distance[i][j]) < 0) {
	fprintf(stderr, "error reading (%ld, %ld) tsp_distance in data file\n", i, j);
	exit(1);	
      }
    }
  }
  VERBOSE ( printf("\n     ....    TSP distance matrix read %ld\n", n_cities); )  
}



void initialize_pheromone_matrix( long int size , double initial_value )
/*    
      FUNCTION:      initialize pheromone matrix to initial_value 
      INPUT:         size of TSP instance, initial value to which pheromones are set.
      OUTPUT:        none (void)
      (SIDE)EFFECTS: assigns initial values to submatrix 0 .. n_cities X 0 .. n_cities of matrix pheromone

*/
{
  long int     i, j;

  for ( i = 0 ; i < size ; i++ ) {
    for ( j = i  ; j < size ; j++ ) {
      pheromone[i][j] = initial_value;  /* symmetric TSP instances; hence phermone[i][j] = pheromone[j][i] */ 
      pheromone[j][i] = initial_value; 
    }
  }
}



void print_distance_matrix ( long int size ) {
/*    
      FUNCTION:      prints an integer matrix m
      INPUT:         dimension of matrix (size)
      OUTPUT:        none
*/
  long int i, j;

  printf("\n");
  for ( i = 0 ; i < size ; i++ ) {
    for ( j = 0 ; j < size ; j++ ) {
      printf(" %ld ", distance[i][j]);
    }
    printf("\n");
  }
}    



void print_pheromone_matrix ( long int size ) {
/*    
      FUNCTION:      prints the pheromone matrix m
      INPUT:         dimension of matrix (size)
      OUTPUT:        none
*/
  long int i, j;

  printf("\n");
  for ( i = 0 ; i < size ; i++ ) {
    for ( j = 0 ; j < size ; j++ ) {
      printf(" %lf ", pheromone[i][j]);
    }
    printf("\n");
  }
}    


long int termination_condition( void )
/*    
      FUNCTION:      checks whether the termination condition is satisfied
      INPUT:         none
      OUTPUT:        TRUE if satisfied, FALSE otherwise
*/
{

  return ( iterations >= max_iterations ); 

}

 

void init_program(  int argc, char *argv[] )
/*    
      FUNCTION:      all the stuff for starting the program: input, parameters, etc.
      INPUT:         argument list from program call
      OUTPUT:        none
*/
{
    FILE *tsp_file;
    long int i;

    setbuf(stdout,NULL);
    printf(PROG_ID_STR);

    if (seed<0) 
	seed = (long int) time(NULL); /* initialize random number generator */

    VERBOSE ( printf("seed %ld\n", seed); ); 

    if( (tsp_file = fopen(argv[1], "r")) == NULL) {
	fprintf(stderr, "error opening input file %s\n",argv[1]);
	exit(1);	
    }

    /* read instance data */
    read_problem_size( tsp_file );

    read_distance_matrix( tsp_file, n_cities );

    print_distance_matrix( n_cities );

    //initialize_pheromone_matrix( n_cities, 0.001 );

    print_pheromone_matrix( n_cities );

    VERBOSE ( printf("end init program\n"); )

}



void start_trial( )
/*    
      FUNCTION:      initializes some stuff when starting a new independent trial
      INPUT:         none
      OUTPUT:        none
*/
{
    long int i;

    VERBOSE( printf("start trial %ld .. \n", trials); )

    iterations = 1;

    initialize_pheromone_matrix( n_cities, 0.001 ); 

}



void pheromone_evaporation( void )
/*    
      FUNCTION:      simulate the pheromone evaporation in each iteration 
      INPUT:         none
      OUTPUT:        none
      (SIDE)EFFECTS: all pheromone trails are reduced by a factor rho
*/
{ 

}

    

void deposit_pheromone( long int *s, long int value )
/*    
      FUNCTION:      deposit pheromone on solution s  
      INPUT:         solution s and its objective function value
      OUTPUT:        none
      (SIDE)EFFECTS: increases pheromone on components of the solution s
*/
{  

}



int main(int argc, char *argv[]) {
  

  /* variables for main programs are declared above */
  

  init_program(argc, argv);   /* initialize all important data */

  /*

  for ( trials = 1 ; trials <= max_trials ; trials ++ ) {

    start_trial();

    while ( !termination_condition() ) {
      
      construct_solutions();

      evaporate_pheromones();

      deposit_pheromones();
      
      iterations++;
	  
    }

    end_trial();

  }

  */

}


